!+
! Module: imgrad_mod
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: This task approximates the gradient at each pixel of an image by doing least-squares fitting of a plane to the 3x3 array of values centred on each pixel. If any of the 3x3 is null then the output is also null.
! 
! Comments: 
! 
! $Id: imgrad_mod.f90,v 1.1 2013/09/06 12:00:44 eojero Exp $
!- 
module imgrad_mod

  use dal
  use param  
  use errorHandling

  use dal_aux
  use save_image

  implicit none 

  private 

  ! Command-line parameters:
  character(255) ::&
    imageSetName = '',&
    xGradImageSetName = '',&
    yGradImageSetName = '',&
    gradMagnitudeImageSetName = '',&
    gradAngleImageSetName = '',&
    tempImageSetName = ''
  character(10) ::&
    outputStyle = ''

  public imgrad

contains 

  !---------------------------------------------------------------------
  subroutine imgrad
  !---------------------------------------------------------------------

    implicit none

    real(single), pointer :: image(:,:)
    integer(int32) :: xsize, ysize, xi, yi
    logical, allocatable :: inPixelIsNull(:,:), outPixelIsNull(:,:)
    type(DataSetT) :: inSet, outSet
    type(ArrayT) :: inArray, outArray
    real(single), allocatable :: gradImage(:,:,:)
logical :: testFlag = .false.

    !-------------------------------------------------------------------
    ! Start

if (testFlag) print *, 'aaa'

    call readParameters
    call readArrayData(imageSetName, image)
    xsize = size(image, 1)
    ysize = size(image, 2)

    allocate(inPixelIsNull(xsize, ysize), outPixelIsNull(xsize, ysize))
if (testFlag) print *, 'bbb'

    ! Store the nulls in the input dataset in inPixelIsNull():
    inSet = dataSet(imageSetName, READ)
    inArray = array(inSet, 0, READ)
    inPixelIsNull = .false. ! default
!    do xi = 1, xsize
!if (testFlag) print *, '  xi:', xi
!      do yi = 1, ysize
!!        if (myIsNullArray(inArray, xi-1, yi-1)) then
!
!!!!if (isNullArray_ims(inArray, xi-1, yi-1)) then
!! This is very slow in low-memory mode (perhaps because it has to haul the dataset from disc at every call). A straight call to isNull was no better, hence for the time being we'll just cut this entire null test out (IMS 2005-06-08).
!
!if (isNull(inArray, yi-1 * xsize + xi-1)) then
!
!          inPixelIsNull(xi, yi) = .true.
!        end if
!      end do
!    end do
    call release(inSet)
if (testFlag) print *, 'ccc'

    ! No gradient is calculated at the edges:
    outPixelIsNull(:,1)     = .true.
    outPixelIsNull(:,ysize) = .true.
    outPixelIsNull(1,:)     = .true.
    outPixelIsNull(xsize,:) = .true.
if (testFlag) print *, 'ddd'

    ! For each 3x3, if one of the member pixels is null, set the output to null:
    outPixelIsNull(2:xsize-1,2:ysize-1)&
      =     inPixelIsNull(1:xsize-2,1:ysize-2)&
      .and. inPixelIsNull(1:xsize-2,2:ysize-1)&
      .and. inPixelIsNull(1:xsize-2,3:ysize  )&
      .and. inPixelIsNull(2:xsize-1,1:ysize-2)&
      .and. inPixelIsNull(2:xsize-1,2:ysize-1)&
      .and. inPixelIsNull(2:xsize-1,3:ysize  )&
      .and. inPixelIsNull(3:xsize  ,1:ysize-2)&
      .and. inPixelIsNull(3:xsize  ,2:ysize-1)&
      .and. inPixelIsNull(3:xsize  ,3:ysize  )

    deallocate(inPixelIsNull)
if (testFlag) print *, 'eee'

    allocate(gradImage(xsize, ysize, 2))
    gradImage = 0.0
if (testFlag) print *, 'fff'

    ! Calculate the components of the gradient:
    do xi = 2, xsize - 1
      do yi = 2, ysize - 1
      if (outPixelIsNull(xi, yi)) cycle

        gradImage(xi, yi, 1)&
          = (sum(image(xi+1,yi-1:yi+1)) - sum(image(xi-1,yi-1:yi+1))) / 6.0
        gradImage(xi, yi, 2)&
          = (sum(image(xi-1:xi+1,yi+1)) - sum(image(xi-1:xi+1,yi-1))) / 6.0
      end do
    end do
if (testFlag) print *, 'ggg'

    select case(outputStyle)
    case('cartesian')
      call saveImage(gradImage(:,:,1), imageSetName&
        , xGradImageSetName, tempImageSetName)

      call saveImage(gradImage(:,:,2), imageSetName&
        , yGradImageSetName, tempImageSetName)

    case('polar')
      call saveImage(sqrt(gradImage(:,:,1) * gradImage(:,:,1)&
        + gradImage(:,:,2) * gradImage(:,:,2)), imageSetName&
        , gradMagnitudeImageSetName, tempImageSetName)

      call saveImage(atan2(gradImage(:,:,2), gradImage(:,:,1)), imageSetName&
        , gradAngleImageSetName, tempImageSetName)
    case default
call error('badOutputStyle',"The value of the --outputstyle parameter was &
&not recognized.")
    end select
if (testFlag) print *, 'hhh'

    deallocate(gradImage)

    ! Store the nulls:
if (testFlag) print *, 'iii'

    select case(outputStyle)
    case('cartesian')
      outSet = dataSet(xGradImageSetName, MODIFY)
    case('polar')
      outSet = dataSet(gradMagnitudeImageSetName, MODIFY)
    case default
call error('badOutputStyle',"The value of the --outputstyle parameter was &
&not recognized.")
    end select
if (testFlag) print *, 'jjj'

    outArray = array(outSet, 0)
!    do xi = 1, xsize
!      do yi = 1, ysize
!        if (outPixelIsNull(xi, yi)) then
!!          call mySetToNullArray(outArray, xi-1, yi-1)
!call setToNullArray_ims(outArray, xi-1, yi-1)
!        end if
!      end do
!    end do
! Very slow in low-memory mode. For the time being we'll just cut this entire null test out (IMS 2005-06-08).
    call release(outSet)
if (testFlag) print *, 'kkk'

    select case(outputStyle)
    case('cartesian')
      outSet = dataSet(yGradImageSetName, MODIFY)
    case('polar')
      outSet = dataSet(gradAngleImageSetName, MODIFY)
    case default
call error('badOutputStyle',"The value of the --outputstyle parameter was &
&not recognized.")
    end select
if (testFlag) print *, 'lll'

    outArray = array(outSet, 0)
!    do xi = 1, xsize
!      do yi = 1, ysize
!        if (outPixelIsNull(xi, yi)) then
!!          call mySetToNullArray(outArray, xi-1, yi-1)
!call setToNullArray_ims(outArray, xi-1, yi-1)
!        end if
!      end do
!    end do
! Very slow in low-memory mode. For the time being we'll just cut this entire null test out (IMS 2005-06-08).
    call release(outSet)
if (testFlag) print *, 'zzz'

  end subroutine imgrad

  !---------------------------------------------------------------------
  subroutine readParameters
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

    imageSetName = stringParameter('imageset')
    outputStyle  = stringParameter('outputstyle')
    select case(outputStyle)
    case('cartesian')
      xGradImageSetName         = stringParameter('xgradimageset')
      yGradImageSetName         = stringParameter('ygradimageset')
    case('polar')
      gradMagnitudeImageSetName = stringParameter('gradsizeimageset')
      gradAngleImageSetName     = stringParameter('gradazimuthimageset')
    case default
call error('badOutputStyle',"The value of the --outputstyle parameter was &
&not recognized.")
    end select

  end subroutine readParameters

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module imgrad_mod
