!
! Module: immask_mod
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: This task makes a mask image which includes only those pixels whose values obey the specified selection expression.
! 
! Comments: 
! 
! $Id: immask_mod.f90,v 1.1 2013/09/06 12:00:44 eojero Exp $
!- 
module immask_mod

  use dal
  use param  
  use errorHandling
  use datafilter

  use dal_aux
  use save_image

  implicit none 

  private 

  ! Command-line parameters:
  character(256) ::&
    imageSetName,&
    tempSetName,&
    tempImageSetName,&
    maskSetName,&
    selectionExpression

  public immask

contains 

  !---------------------------------------------------------------------
  subroutine immask
  !---------------------------------------------------------------------

    implicit none

    type(DataSetT) :: inSet, tempSet
    type(ArrayT) :: inArray
    type(TableT) :: tempTab
    type(ColumnT) :: tempCol, flagCol
    integer(int32), pointer :: inDimVector(:)
    integer(int32) :: inDataType, imageShape(0:1), numSelectedPixels
    integer(int8),  pointer :: inDataInt8(:,:),   tempDataInt8(:), flagData(:)
    integer(int16), pointer :: inDataInt16(:,:),  tempDataInt16(:)
    integer(int32), pointer :: inDataInt32(:,:),  tempDataInt32(:)
    real(single),   pointer :: inDataReal32(:,:), tempDataReal32(:)
    real(double),   pointer :: inDataReal64(:,:), tempDataReal64(:)

    !-------------------------------------------------------------------
    ! Start

    call readParameters

    inSet = dataSet(imageSetName, READ)
    inArray = array(inSet, 0)
    inDimVector => dimensions(inArray)

    imageShape = inDimVector ! This is done so shape information is
    ! retained even after inArray is released to conserve memory.

    inDataType = dataType(inArray)

    tempSet = dataSet(tempSetName, TEMP)
    tempTab = addTable(tempSet, 'TEMP_TAB', inDimVector(0)*inDimVector(1))
    tempCol = addColumn(tempTab, 'VAL', inDataType)
    flagCol = addColumn(tempTab, 'FLAG', INTEGER8)
    flagData => int8Data(flagCol)
    flagData = 0
    call release(flagCol)
    call initFilter(selectionExpression)

    select case(inDataType)
    case(INTEGER8)
      if (nullDefined(inArray)) then
        call setNullValue(tempCol, intNullValue(inArray))
      end if

      inDataInt8   => int8Array2Data(inArray)
      tempDataInt8 => int8Data(tempCol)
      tempDataInt8 = reshape(inDataInt8, shape(tempDataInt8))

    case(INTEGER16)
      if (nullDefined(inArray)) then
        call setNullValue(tempCol, intNullValue(inArray))
      end if

      inDataInt16 => int16Array2Data(inArray)
      tempDataInt16 => int16Data(tempCol)
      tempDataInt16 = reshape(inDataInt16, shape(tempDataInt16))

    case(INTEGER32)
      if (nullDefined(inArray)) then
        call setNullValue(tempCol, intNullValue(inArray))
      end if

      inDataInt32 => int32Array2Data(inArray)
      tempDataInt32 => int32Data(tempCol)
      tempDataInt32 = reshape(inDataInt32, shape(tempDataInt32))

    case(REAL32)
      inDataReal32  => real32Array2Data(inArray)
      tempDataReal32 => real32Data(tempCol)
      tempDataReal32 = reshape(inDataReal32, shape(tempDataReal32))

    case(REAL64)
      inDataReal64 => real64Array2Data(inArray)
      tempDataReal64 => real64Data(tempCol)
      tempDataReal64 = reshape(inDataReal64, shape(tempDataReal64))

!    case(BOOLEAN)
!      inDataBool => boolArray2Data(inArray)

    case default
call error('badArrayDataType',"The data type of the input image was &
&not recognized.")
    end select

    call release(inSet)
    call release(tempCol)
    numSelectedPixels = doSelectModify(tempTab, 'FLAG', 0_int16)
    flagCol = column(tempTab, 'FLAG')
    flagData => int8Data(flagCol)

    call saveImage(reshape(flagData, imageShape), imageSetName&
      , maskSetName, tempImageSetName) ! in ssclib/save_image.f90

    call release(tempSet)

  end subroutine immask

  !---------------------------------------------------------------------
  subroutine readParameters
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

    imageSetName        = stringParameter('imageset')
    tempSetName         = stringParameter('tempset')
    tempImageSetName    = stringParameter('tempimageset')
    maskSetName         = stringParameter('maskset')
    selectionExpression = stringParameter('expression')

  end subroutine readParameters

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module immask_mod
