!+
! Module: immedian_mod
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: The task performs a median smoothing of an image.
! 
! Comments: 
! 
! $Id: immedian_mod.f90,v 1.1 2013/09/06 12:00:44 eojero Exp $
!- 
module immedian_mod

  use dal
  use param  
  use errorHandling

  use array_utils
  use dal_aux
  use save_image

  implicit none 

  private 

  ! Command-line parameters:
  character(256) ::&
    inSetName='',&
    outSetName='',&
    tempImageSetName='',&
    medianMaskSetName=''
  character(20) ::&
    maskStyle=''
  integer(int16) ::&
    maskXiSize=0,&
    maskYiSize=0,&
    maskRadiusI=0

  public immedian

contains 

  !---------------------------------------------------------------------
  subroutine immedian
  !---------------------------------------------------------------------

    implicit none

    real(single), pointer :: inImage(:,:)
!    logical(bool), pointer :: medianMask(:,:)
    logical, pointer :: medianMask(:,:)
    integer(int16) ::&
      xiSize, yiSize, xi, yi, dxI, dyI,&
      halfMaskXiSize, halfMaskYiSize, maskRadiusISqu, radiusISqu
    real(single), allocatable :: outImage(:,:)
    integer(int16), allocatable ::&
      imXiStart(:), maskXiStart(:), imYiStart(:), maskYiStart(:),&
      imXiFinis(:), maskXiFinis(:), imYiFinis(:), maskYiFinis(:)

    !-------------------------------------------------------------------
    ! Start

    call readParameters

    call readArrayData(inSetName, inImage)
    xiSize = size(inImage, 1)
    yiSize = size(inImage, 2)
    allocate(outImage(xiSize, yiSize))

    select case(maskStyle)
    case('rectangle')
      allocate(medianMask(maskXiSize, maskYiSize))
      medianMask = .true.
      halfMaskXiSize = floor(maskXiSize / 2.0)
      halfMaskYiSize = floor(maskYiSize / 2.0)

    case('round')
      maskXiSize = 1 + 2 * maskRadiusI
      maskYiSize = 1 + 2 * maskRadiusI
      halfMaskXiSize = floor(maskXiSize / 2.0)
      halfMaskYiSize = floor(maskYiSize / 2.0)
      maskRadiusISqu = maskRadiusI * maskRadiusI

      allocate(medianMask(maskXiSize, maskYiSize))
      medianMask = .true. ! default
      do xi = 1, maskXiSize
        dxI = xi - maskRadiusI - 1
        do yi = 1, maskYiSize
          dyI = yi - maskRadiusI - 1
          radiusISqu = dxI * dxI + dyI * dyI
          if (radiusISqu > maskRadiusISqu) medianMask(xi,yi) = .false.
        end do
      end do

    case('set')
      call readArrayData(medianMaskSetName, medianMask)
      maskXiSize = size(medianMask,1)
      maskYiSize = size(medianMask,1)
      halfMaskXiSize = floor(maskXiSize / 2.0)
      halfMaskYiSize = floor(maskYiSize / 2.0)

    case default
call error('',"")
    end select

if (.not.any(medianMask)) call error('',"")
    if (count(medianMask)==1) call warning('',"")

    allocate(imXiStart(xiSize), maskXiStart(xiSize))
    allocate(imYiStart(yiSize), maskYiStart(yiSize))
    allocate(imXiFinis(xiSize), maskXiFinis(xiSize))
    allocate(imYiFinis(yiSize), maskYiFinis(yiSize))

    do xi = 1, xiSize
      imXiStart(xi) = max(1,      xi-(maskXiSize-1-halfMaskXiSize))
      imXiFinis(xi) = min(xiSize, xi              +halfMaskXiSize)
      maskXiStart(xi) = max(1, (maskXiSize-halfMaskXiSize)-(xi-1))
      maskXiFinis(xi) = min(maskXiSize&
        , (maskXiSize-halfMaskXiSize)+(xiSize-xi))
    end do
    do yi = 1, yiSize
      imYiStart(yi) = max(1,      yi-(maskYiSize-1-halfMaskYiSize))
      imYiFinis(yi) = min(yiSize, yi              +halfMaskYiSize)
      maskYiStart(yi) = max(1, (maskYiSize-halfMaskYiSize)-(yi-1))
      maskYiFinis(yi) = min(maskYiSize&
        , (maskYiSize-halfMaskYiSize)+(yiSize-yi))
    end do

    do xi = 1, xiSize
      do yi = 1, yiSize
        outImage(xi,yi) = median(inImage(imXiStart(xi):imXiFinis(xi)&
                                        ,imYiStart(yi):imYiFinis(yi))&
          , logical(medianMask(maskXiStart(xi):maskXiFinis(xi)&
                              ,maskYiStart(yi):maskYiFinis(yi))))
      end do
    end do

    call saveImage(outImage, inSetName, outSetName, tempImageSetName)

    deallocate(inImage, medianMask)

  end subroutine immedian

  !---------------------------------------------------------------------
  subroutine readParameters
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

    inSetName = stringParameter('inset')
    outSetName = stringParameter('outset')
    tempImageSetName = stringParameter('tempset')
    maskStyle = stringParameter('maskstyle')
    select case(maskStyle)
    case('rectangle')
      maskXiSize = intParameter('xsize')
      maskYiSize = intParameter('ysize')

    case('round')
      maskRadiusI = intParameter('radius')

    case('set')
      medianMaskSetName = stringParameter('maskset')

    case default
call error('',"")
    end select

  end subroutine readParameters

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module immedian_mod
