!+
! Module: impoissprob_mod
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: Given a Poisson counts image and an expectation image, this calculates, for each image pixel, the probability of one arising from the other.
! 
! Comments: 
! 
! $Id: impoissprob_mod.f90,v 1.1 2013/09/06 12:00:44 eojero Exp $
!- 
module impoissprob_mod

  use dal
  use param  
  use errorHandling

  use dal_aux
  use detection_stats
  use math_utils
  use save_image

  implicit none 

  private 

  ! Command-line parameters:
  character(256) ::&
    countsImageSetName='',&
    expectationImageSetName='',&
    outImageSetName='',&
    tempImageSetName=''
  logical ::&
    doIntegratedProbGeCts=.false.,&
    showOutputAsLikelihoods=.false.

  public impoissprob

contains 

  !---------------------------------------------------------------------
  subroutine impoissprob
  !---------------------------------------------------------------------

    implicit none

    integer(int32), pointer :: countsImage(:,:)
    real(single), pointer :: expectationImage(:,:)
    integer :: xiSize, yiSize, xi, yi
    real(single), allocatable :: outImage(:,:)
    logical, allocatable :: nulls(:,:)
    integer :: foreignStatus
    type(DataSetT) :: set
    type(ArrayT) :: inArray

    !-------------------------------------------------------------------
    ! Start

    call readParameters

    call readArrayData(countsImageSetName, countsImage) ! in ssclib/dal_aux.
if (any(countsImage < 0)) call error('',"")
    xiSize = size(countsImage, 1)
    yiSize = size(countsImage, 2)
    call readArrayData(expectationImageSetName, expectationImage)
    ! in ssclib/dal_aux.

    if ( size(expectationImage,1) .ne. xiSize&
    .or. size(expectationImage,2) .ne. yiSize)&
call error('',"")

    allocate(outImage(xiSize, yiSize), nulls(xiSize, yiSize))

    outImage = 0.0
    nulls = .false.
    if (doIntegratedProbGeCts) then
      do xi = 1, xiSize
        do yi = 1, yiSize
          if (expectationImage(xi,yi) < 0.0&
          .or. (expectationImage(xi,yi) == 0.0&
          .and. countsImage(xi,yi) > 0)) then
            nulls(xi,yi) = .true.
!            call setToNullArray_ims(outImage, xi, yi) ! in ssclib/dal_aux.

          else
            call incompleteGammaP(real(countsImage(xi,yi))&
              , expectationImage(xi,yi), outImage(xi,yi), foreignStatus)
            ! in ssclib/math_utils.
            if (foreignStatus .ne. 0) then
print *, 'incompleteGammaP status was ', foreignStatus
call error('',"")
            end if
          end if
        end do
      end do

    else
      do xi = 1, xiSize
        do yi = 1, yiSize
          if (expectationImage(xi,yi) < 0.0&
          .or. (expectationImage(xi,yi) == 0.0&
          .and. countsImage(xi,yi) > 0)) then
            nulls(xi,yi) = .true.
!            call setToNullArray_ims(outImage, xi, yi) ! in ssclib/dal_aux.

          else
            call incompleteGammaQ(real(countsImage(xi,yi))&
              , expectationImage(xi,yi), outImage(xi,yi), foreignStatus)
            ! in ssclib/math_utils.
            if (foreignStatus .ne. 0) then
print *, 'incompleteGammaQ status was ', foreignStatus
call error('',"")
            end if
          end if
        end do
      end do
    end if

    if (showOutputAsLikelihoods) then
      where(.not.nulls .and. outImage > 0.0) outImage = -log(outImage)
    end if

    call saveImage(outImage, countsImageSetName, outImageSetName&
      , tempImageSetName)

    deallocate(countsImage, expectationImage)

    set = dataSet(outImageSetName, MODIFY)
    inArray = array(set, 0)
    do xi = 1, xiSize
      do yi = 1, yiSize
        if (nulls(xi,yi)) call setToNullArray_ims(inArray, xi, yi)
      end do
    end do

    call release(set)

  end subroutine impoissprob

  !---------------------------------------------------------------------
  subroutine readParameters
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

    countsImageSetName      = stringParameter('poissset')
    expectationImageSetName = stringParameter('expectset')
    outImageSetName         = stringParameter('outset')
    doIntegratedProbGeCts   = booleanParameter('gecounts')
    showOutputAsLikelihoods = booleanParameter('showlike')
    tempImageSetName        = stringParameter('tempset')

  end subroutine readParameters

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module impoissprob_mod
