//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	orbitcmp --- CMPare ODF ORBIT data with TLE data from NASA/OIG
//
//	DESCRIPTION:
//		For a given observation specified as ODF pointed at via SAS_ODF
//		the task compares the satellite's position and velocity as
//		obtained from the reconstructed or predicted orbit file
//		(see SAS_ORBIT) and the XmmOrbitSimulator in package caloalutils
//		which uses Two-Line-Element (TLE) orbital data from NASA's OIG
//		(http://oig1.gsfc.nasa.gov).
//
//		The comparison is made over the time period covered by the ODF
//		orbit file with a granularity specified via task parameter
//		<tstep> (d/f: 10s). Results are printed to stdout in the format:
//
//			t ds dv
//
//		where
//			t			: time in Mission-Elapsed-Time [s]
//			ds			: difference between satellite position calculated
//						  from ODF and TLE orbit data in units of light-ms
//			dv			: difference between satellite velocity calculated
//						  from ODF and TLE orbit data in units of [m/s]
//
//		Parameter <offsett>=true causes all time tags to be offset by time of
//		first element which, hence, is assigned	t=0.
//
//	USAGE:
//		orbitcmp [tstep=<value>] [offsett=true|false] >orbitcmp.dat
//
//	CHANGE HISTORY:
//		UL, 2003-04-25: 1.0.0: - initial version
//
//	AUTHOR:
//		U. Lammers, Fri Apr 25 15:27:55 MEST 2003
//
//	$HEADER$
//
#include <iostream>
#include <iomanip>

#include "OalServer.h"
#include "XmmPositionLocator.h"
#include "Xmm.h"
#include "param.h"
#include "STime.h"
#include "PhysicalConstants.h"

#include "orbitcmp.h"

void orbitcmp()
{
	const XmmPositionLocator *actLoc = oalServer.positionLocator();

	STime tFrom = actLoc->dataBegin(),
		  tTo = actLoc->dataEnd();
	double deltaT = realParameter("tstep");		// [s]
	unsigned N = UNT((tTo-tFrom)/deltaT);
	bool offset = booleanParameter("offsett");

	double t0 = tFrom - Xmm::missionReferenceTime;
	for (unsigned n=0; n<N; ++n) {
		StateVector svODF, svTLE;
		double t = t0+DBL(n)*deltaT;
		try {
			svODF = actLoc->stateVector(t);
		} catch(XmmPositionLocator::NoPositionDataAvailable) {
			errstr<<"No position data available at t="
				 <<t<<Sas::warning("NoPositionData");
			continue; // inore this point
		}
		XmmOrbitSimulator *simLoc = new XmmOrbitSimulator(tFrom+DBL(n)*deltaT);
		svTLE.position = simLoc->position(t);
//cout<<"PosAct: "<<svODF.position<<endl;
//cout<<"PosSim: "<<svTLE.position<<endl;
		svTLE.velocity = simLoc->velocity(t);
		delete simLoc;

		double ds = (svODF.position - svTLE.position).norm();
//					PhysicalConstants::c,
//			   dv = (svODF.velocity - svTLE.velocity).norm()/1000.;
		cout<<setprecision(16)<<(offset ? t-t0 : t)<<' '
			<<setprecision(8)<<ds<<endl;
//			<<setprecision(8)<<dv<<endl;
	}
} // orbitcmp()

