//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	satvelsrc --- compute SATellite VELocities in direction of specified source
//
//	DESCRIPTION:
//		Given an ODF (pointed at with SAS_ODF) print as a function of
//		time the value of the component of the S/C velocity vector
//		in the direction of the source being pointed at. The time
//		series begins at the observation start time (as obtained from
//		the ODF) through the actual end time in steps of <tdelt> (command
//		line parameter). The position of the source is taken from the
//		ODF but can be overridden via command line parameters
//		withsrccoord/srcra/srcdec.
//
//		Output data are simply written as
//			t [d-INT[d]]	vel [km/s]
//		to stdout. E.g. use
//			SAS_ODF=.... && satvelsrc | xmgrace
//		to generate a time series plot.
//
//	CHANGE HISTORY:
//		UL, 2002-03-20: 1.0.0: - initial version
//
//	AUTHOR:
//		U. Lammers, Wed Mar 20 16:28:32 MET 2002
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/tools/src/satvelsrc/satvelsrc.cc,v 1.7 2013/09/05 14:51:50 eojero Exp $
//

#include <cmath>

#include "common.h"
#include "OalServer.h"
#include "Xmm.h"
#include "Celestial.h"
#include "STime.h"

#include "satvelsrc.h"

void satvelsrc()
{
	const Odf *odf = oalServer.odf();
	STime t0(odf->observationStartTime()),
		  t1(odf->observationEndTime());

	if (booleanParameter("withexplicittimes")) {
		t0 = STime(stringParameter("tfrom"));
		t1 = STime(stringParameter("tto"));
	}
	double obsLen = t1 - t0,
		   delta = realParameter("tdelt");
	STime::JulianDayNumber t0jd = t0.modifiedJulianDay();
	double t0jdint = t0jd.day(),
		   t0jdfull = t0jd.full();
	if (booleanParameter("withmjdoffset"))
		t0jdint = intParameter("mjdoffset");

	const XmmPositionLocator *posLoc = new XmmPositionLocator(
						odf->selectSet(Xmm::SC, Xmm::RECONSTRUCTED_ORBIT, 1));

	//	obtain unit vector to source
	RightAscension ra(odf->proposalInfo().targetRa * DEG_TO_RAD);
	Declination dec(odf->proposalInfo().targetDec * DEG_TO_RAD);

	if (booleanParameter("withorthosrc")) {
		//	assume source is orthogonal to orbit plane; construct plane
		//	from the three position vectors at t0, t0+obsLen/2, t0+obsLen
		Plane orbitPlane(posLoc->stateVector(t0).position,
						 posLoc->stateVector(t0+.5*obsLen).position,
						 posLoc->stateVector(t0+obsLen).position);
		EquatorialDirection src(orbitPlane.normal());
		ra = src.ra();
		dec = src.dec();
	} else if (booleanParameter("withsrccoord")) {
		ra = RightAscension(stringParameter("srcra"));
		dec = Declination(stringParameter("srcdec"));
	}
	CartesianVector srcVec = EquatorialDirection(ra, dec).toUnitCartesian();

	//	print out a few data about ODF and loop
	cerr<<"Revolution    : "<<odf->orbitNr()<<endl;
	cerr<<"Observation ID: "<<odf->proposalId()+odf->observationId()<<endl;
	cerr<<"Start time    : "<<t0<<endl;
	cerr<<"End time      : "<<t1<<endl;
	cerr<<"Duration      : "<<obsLen<<"[s]"<<endl;
	cerr<<"Target        : "<<odf->proposalInfo().targetName<<endl;
	cerr<<"Printing satellite velocities projected onto vector "
		<<srcVec<<" to source at: "
		<<ra<<'/'<<dec<<" offset by MJD="<<t0jdint<<endl;

	for (unsigned n=0; DBL(n)*delta <= obsLen; ++n) {
		double t = DBL(n)*delta,
			   velsrc = 0.;
		try {
			CartesianVector v = posLoc->stateVector(t0 + t).velocity;
			velsrc = v * srcVec;	//	velocity of S/C in direction of source
									//	[km/s]
		} catch (...) {
			continue;	//	no position data - skip this time
		}
		cout<<t0jdfull+t/STime::secondsPerDay-t0jdint<<' '<<velsrc<<endl;
	}
} // satvelsrc()
